use crate::middlewares::error::ErrorResponse;
use crate::models::playlists::Playlists;
use crate::models::user::Users;
use actix_web::http::StatusCode;
use actix_web::{get, web, HttpResponse, Result, Scope};
use diesel::result::Error as DBError;
use serde::Serialize;

pub fn routes() -> Scope {
    web::scope("/users")
        .service(get_user)
        .service(get_user_playlists)
}

#[derive(Serialize)]
struct GetUserResponse {
    id: String,
    name: String,
    cover: String,
}

fn get_a_user(user_id: &str) -> Result<Users, ErrorResponse> {
    let user = Users::find(user_id);

    match user {
        Ok(user) => Ok(user),
        Err(DBError::NotFound) => Err(ErrorResponse::new("User not found", StatusCode::NOT_FOUND)),
        _ => Err(ErrorResponse::new(
            "Unknown error",
            StatusCode::INTERNAL_SERVER_ERROR,
        )),
    }
}

#[get("/{user_id}")]
async fn get_user(path: web::Path<String>) -> Result<HttpResponse, ErrorResponse> {
    let user = get_a_user(&path.into_inner())?;
    let user_id = &user.id;

    Ok(HttpResponse::Ok().json(GetUserResponse {
        id: user_id.to_string(),
        name: user.name,
        cover: format!("https://assets.vybr.net/users/{}.png", user_id),
    }))
}

#[get("/{user_id}/playlists")]
async fn get_user_playlists(path: web::Path<String>) -> Result<HttpResponse, ErrorResponse> {
    let user = get_a_user(&path.into_inner())?;

    let playlists = Playlists::find_for_user(&user.id, true)?;

    Ok(HttpResponse::Ok().json(playlists))
}
